var template = null;

const templateTypes = {
	"bottom-slogan": BottomSloganTemplate,
	"bottom-slogan-gradient": BottomSloganGradient,
	"bottom-slogan-gradient-icon": BottomSloganGradientIcon,
	"bottom-slogan-gradient-vertical-logo": BottomSloganGradientVerticalLogo,
	"bottom-slogan-person": BottomSloganPerson,
	"bottom-slogan-quote": BottomSloganQuote,
	"bottom-slogan-full-width": BottomSloganFullWidth,
	"bottom-slogan-vertical-logo": BottomSloganVerticalLogo,
	"left-long-text": LeftLongTextTemplate,
	"right-long-text": RightLongTextTemplate,
	"right-big-text": RightBigTextTemplate,
	"left-corner-slogan": LeftCornerSloganTemplate,
	"story-with-rectangle": StoryWithRectangleTemplate,
	"story-with-full-width-rectangle": StoryWithFullWidthRectangleTemplate,
	"no-image-icon-text": NoImageIconText,
	"no-image-text-more-info": NoImageTextMoreInfo,
	"no-image-quote": NoImageQuote,
	"no-image-big-text-icon-with-terciary": NoImageBigTextIconWithTerciary,
	"no-image-big-text-icon": NoImageBigTextIcon,
	"no-image-text-icon-vertical-logo": NoImageTextIconVerticalLogo,
	"banner-name-left": BannerNameLeft,
	"banner-name-right": BannerNameRight,
	"cover": Cover,
	"cover-static-name-and-logo": CoverStaticNameAndLogo,
	"cover-static-name-and-logo-left": CoverStaticNameAndLogoLeft,
	"poll": Poll,
	"event-text-right": EventTextRight,
	"event-text-bottom": EventTextBottom
}

const attributeElements = {
	"primaryImage": "#primary-image-wrapper, #primary-image-position-wrapper-lower",
	"additionalPrimaryImages3": "#additional-3-primary-images-wrapper",
	"locationImage": "#location-image-wrapper",
	"iconImage": "#icon-image-wrapper",
	"primaryText": "#primary-text-wrapper",
	"secondaryText": "#secondary-text-wrapper",
	"terciaryText": "#terciary-text-wrapper",
	"urlText": "#url-text-wrapper",
	"underNameText": "#under-name-text-wrapper",
	"secondaryNameText": "#secondary-name-text-wrapper",
	"secondaryUnderNameText": "#secondary-under-name-text-wrapper",
	"nameText": "#name-text-wrapper",
	"iconText": "#icon-text-wrapper",
	"primaryColorScheme": "#primary-color-scheme-wrapper",
	"nameColorScheme": "#name-color-scheme-wrapper",
	"secondaryColorScheme": "#secondary-color-scheme-wrapper",
	"terciaryColorScheme": "#terciary-color-scheme-wrapper",
	"primaryImagePosition": "#primary-image-position-wrapper",
	"logoIsCenter": "#logo-is-center-wrapper",
	"backgroundHasPattern": "#background-has-pattern-wrapper",
	"qrCode": "#qr-code-wrapper",
	"fourIconSet": "#4-icon-group-wrapper",
	"fiveIconSet": "#5-icon-group-wrapper",
	"fivePointSet": "#5-point-set-wrapper",
	"twoReactionSet": "#2-reaction-set-wrapper",
	"dateText": "#date-text-wrapper",
	"timeText": "#time-text-wrapper",
	"locationText": "#location-text-wrapper",
	"secondaryImage": "#secondary-image-wrapper",
	"gradient": "#gradient-wrapper",
	"logoImage": "#logo-image-wrapper"
}

const colorPickers = {
	"primaryTextColor": "primary-text-color-wrapper",
	"iconColor": "icon-color-wrapper",
	"backgroundColor": "background-color-wrapper",
	"foregroundColor": "foreground-color-wrapper",
	"nameBackgroundColor": "name-background-color-wrapper",
	"nameTextColor": "name-text-color-wrapper",
	"dateTextColor": "date-text-color-wrapper",
	"urlTextColor": "url-text-color-wrapper",
	"primaryTextHighlightColor": "primary-text-highlight-color-wrapper",
	"secondaryTextBackgroundColor": "secondary-text-background-color-wrapper",
	"secondaryTextColor": "secondary-text-color-wrapper",
	"secondaryTextHighlightColor": "secondary-text-highlight-color-wrapper",
	"terciaryTextColor": "terciary-text-color-wrapper",
	"terciaryTextBackgroundColor": "terciary-text-background-color-wrapper",
	"reactionTextColor": "reaction-text-color-wrapper",
	"underNameTextColor": "under-name-text-color-wrapper",
	"qrCodeColor": "qr-code-color-wrapper",
	"requesterTextColor": "requester-text-color-wrapper",
	"quoteColor": "quote-color-wrapper",
	"backgroundGradientColor1": "background-gradient-color1-wrapper",
	"backgroundGradientColor2": "background-gradient-color2-wrapper"
}

function reloadColorPalette() {
	for (let [attribute, elementId] of Object.entries(colorPickers)) {
		if (template.changeableColors.includes(attribute) && template[attribute] !== undefined) {
			$(`#${elementId}`).css("display", "flex");
			$(`#${elementId.replace("-wrapper", "")}`).val(template[attribute].toUpperCase());
			$(`#${elementId.replace("-wrapper", "")}`).trigger("change");
		} else {
			$(`#${elementId}`).css("display", "none");
		}
	}
}

async function initTemplate(templateName, skipSetting = false) {
	if (templateName.startsWith("cover")) {
		$(".guide-left").css("display", "block");
		$(".guide-top").css("display", "block");
		
		if (!templateName.endsWith("left")) {
			document.getElementById("guide-top-image").src = "static/images/cover_guide_top.png";
		} else {
			document.getElementById("guide-top-image").src = "static/images/cover_guide_top-left.png";
		}
	} else {
		$(".guide-left").css("display", "none");
		$(".guide-top").css("display", "none");
	}
	
	if (!skipSetting) {
		template = new templateTypes[templateName]("graphicsCanvas");
	}

	$("#template-description").html(template.description);
	
	for (let [attribute, elementSelector] of Object.entries(attributeElements)) {
		if (template.changeableAttributes.includes(attribute)) {
			$(`${elementSelector}`).css("display", "block");
		} else {
			$(`${elementSelector}`).css("display", "none");
		}
	}
	
	if (template.isPrintable) {
		$("#printing-wrapper").css("display", "block");
	} else {
		$("#printing-wrapper").css("display", "none");
	}
	
	if (document.getElementById("logo-additional-image").files.length !== 0) {
		template.setLogoImageFromInput(document.getElementById("logo-additional-image"));
	}
	
	await template.loadData(
		(
			(template.primaryImage === null) ?
			document.getElementById("primary-image") : null
		),
		$("#primary-text").val(),
		$("#name-text").val(),
		template.primaryColorSchemes[0],
		(
 			(parseInt($("#resolution").val()) === NaN || parseInt($("#resolution").val()) < 750) ?
			parseInt($("#resolution").val()) :
			template.defaultResolution
		),
		true
	);
	
	if (template.changeableAttributes.includes("locationImage")) {
		template.setLocationSource(
			"static/images/locations/Louny.png",
			true
		);
	}
	
	if (template.changeableAttributes.includes("locationImageToTop")) {
		template.setLocationToTop($("#location-to-top").is(":checked"), true);
	}
	
	if (template.changeableAttributes.includes("iconImage")) {
		if ($("#icon-image-input").val() !== "") {
			await template.setIconImageFromInput(document.getElementById("icon-image-input"));
		} else {
			template.setIconSource(
				(
					(
						$("#icon-image-selection").select2("data").length !== 0 &&
						$("#icon-image-selection").select2("data")[0].element.dataset.imageIconSource !== undefined
					) ?
					$("#icon-image-selection").select2("data")[0].element.dataset.imageIconSource :
					null
				),
				true
			);
		}
	}
	
	if (template.changeableAttributes.includes("iconText")) {
		template.setIconText($("#icon-text").val(), true);
	}
	
	if (template.changeableAttributes.includes("secondaryImage")) {
		await template.setSecondaryImageFromInput(document.getElementById("secondary-image"), true);
	}
	
	if (template.changeableAttributes.includes("logoIsCenter")) {
		template.setLogoIsCenter(
			(
				($("#logo-is-center").is(":checked")) ?
				true : false
			),
			true
		);
	}
	
	if (template.changeableAttributes.includes("gradient")) {
		template.setGradient($("#show-gradient").is(":checked"));
	}
	
	if (template.changeableAttributes.includes("backgroundHasPattern")) {
		template.setBackgroundHasPattern(
			(
				($("#background-has-pattern").is(":checked")) ?
				true : false
			),
			true
		);
	}
	
	if (template.changeableAttributes.includes("secondaryText")) {
		template.setSecondaryText(
			$("#secondary-text").val(),
			true
		);
	}
	
	if (template.changeableAttributes.includes("terciaryText")) {
		template.setTerciaryText(
			$("#terciary-text").val(),
			true
		);
	}
	
	if (template.changeableAttributes.includes("urlText")) {
		template.setUrlText(
			$("#url-text").val(),
			true
		);
	}
	
	if (template.changeableAttributes.includes("underNameText")) {
		template.setUnderNameText(
			$("#under-name-text").val(),
			true
		);
	}
	
	if (template.changeableAttributes.includes("secondaryNameText")) {
		template.setSecondaryNameText(
			$("#secondary-name-text").val(),
			true
		);
	}
	
	if (template.changeableAttributes.includes("secondaryUnderNameText")) {
		template.setSecondaryUnderNameText(
			$("#secondary-under-name-text").val(),
			true
		);
	}
	
	if (template.changeableAttributes.includes("dateText")) {
		template.setDateText(
			$("#date-text").val(),
			true
		);
	}
	
	if (template.changeableAttributes.includes("timeText")) {
		template.setTimeText(
			$("#time-text").val(),
			true
		);
	}
	
	if (template.changeableAttributes.includes("locationText")) {
		template.setLocationText(
			$("#location-text").val(),
			true
		);
	}
	
	if ($("#requester-text").css("display") === "block") {
		template.setRequesterText($("#requester-text").val(), true);
	}
	
	if (template.changeableAttributes.includes("qrCode")) {
		template.setQrCodeURI($("#qr-code-source").val(), true);
	}
	
	if (template.changeableAttributes.includes("additionalPrimaryImages3")) {
		for (const num of Array(3).keys()) {
			const sourceElement = $(`#additional-primary-image-${num}`);
			
			$(`#reset-additional-primary-image-${num}`).on(
				"click",
				function() {
					sourceElement.val(null);
					template.additionalPrimaryImages[num] = null;
					template.redrawCanvas();
				}
			)
			
			sourceElement.on(
				"change",
				function (event) {
					if (event.target.files.length === 0) {
						return;
					}
					
					const uploadedImage = new Image();
					
					uploadedImage.onload = function() {
						if (this.width < 1080 || this.height < 1080) {
							alert("Prosím, nahraj obrázek s minimálním rozlišením 1080 pixelů na výšku i šířku.");
							$(event.target).val(null);
							
							return;
						}
						
						template.additionalPrimaryImages[num] = this;
						template.redrawCanvas();
					}
					
					uploadedImage.src = window.URL.createObjectURL(event.target.files[0]);
				}
			);
			
			if (sourceElement[0].files.length !== 0) {
				await template.setAdditionalPrimaryImageFromInput(
					num,
					sourceElement[0],
					true
				);
			}
		}
	}
	
	if (template.changeableAttributes.includes("fourIconSet")) {
		for (const num of Array(4).keys()) {
			// Check input first
			const inputElement = $(`#4-icon-group-image-${num + 1}-input`);
			
			if (inputElement[0].files.length !== 0) {
				inputReadPromise = new Promise(
					resolve => {
						const reader = new FileReader();
						
						reader.onloadend = function() {
							template.iconSet[num].source = reader.result;
							
							resolve();
						};
						
						reader.readAsDataURL(inputElement[0].files[0]);
					}
				);
				
				await inputReadPromise;
				return;
			}
			
			// Selections
			const selectionElement = $(`#4-icon-group-image-${num + 1}-selection`);
			
			if (
				selectionElement.select2("data").length !== 0 &&
				selectionElement.select2("data")[0].element.dataset.imageIconSource !== undefined
			) {
				template.iconSet[num].source = selectionElement.select2("data")[0].element.dataset.imageIconSource;
			}
			
			const textElementValue = $(`#4-icon-group-text-${num + 1}`).val();
			
			if (textElementValue !== "") {
				template.iconSet[num].text = textElementValue;
			}
		}
	}
	
	if (template.changeableAttributes.includes("fivePointSet")) {
		for (const num of Array(5).keys()) {
			const value = $(`#5-point-set-text-${num + 1}`).val();
			
			if (value !== "" && value !== undefined && value !== null) {
				template.pointSet[num] = value;
			}
		}
	}
	
	if (template.changeableAttributes.includes("fiveIconSet")) {
		for (let num of Array(5).keys()) {
			const sourceElement = $(`#5-icon-group-image-${num + 1}-selection`);
			
			if (
				sourceElement.select2("data").length !== 0 &&
				sourceElement.select2("data")[0].element.dataset.imageIconSource !== undefined
			) {
				template.iconSet[num].source = sourceElement.select2("data")[0].element.dataset.imageIconSource;
			}
			
			const textElementValue = $(`#5-icon-group-text-${num + 1}`).val();
			
			if (textElementValue !== "") {
				template.iconSet[num].text = textElementValue;
			}
		}
	}
	
	if (template.changeableAttributes.includes("twoReactionSet")) {
		for (let num of Array(2).keys()) {
			const sourceElement = $(`#2-reaction-set-image-${num + 1}-selection`);
			
			if (
				sourceElement.select2("data").length !== 0 &&
				sourceElement.select2("data")[0].element.dataset.imageReactionSource !== undefined
			) {
				template.reactions[num].source = sourceElement.select2("data")[0].element.dataset.imageReactionSource;
			}
			
			const textElementValue = $(`#2-reaction-set-text-${num + 1}`).val();
			
			if (textElementValue !== "") {
				template.reactions[num].text = textElementValue;
			}
		}
	}
	
	if (
		$("#coalition-selection").select2("data").length !== 0 &&
		$("#coalition-selection").select2("data")[0].id !== undefined &&
		$("#coalition-selection").select2("data")[0].id !== "no-coalition"
	) {
		await setCoalition($("#coalition-selection").select2("data")[0].id, template);
	} else {
		await template.redrawCanvas();
	}
	
	reloadColorPalette();
}

$(".icon-selection").html(`
<option>Žádná ikona</option>
<option
	data-image-source="static/images/icons/Bezpecnost-1-dark.png"
	data-image-icon-source="static/images/icons/Bezpecnost-1.png"
>Bezpečnost 1</option>
<option
	data-image-source="static/images/icons/Bezpecnost-2-dark.png"
	data-image-icon-source="static/images/icons/Bezpecnost-2.png"
>Bezpečnost 2</option>
<option
	data-image-source="static/images/icons/Bydleni-1-dark.png"
	data-image-icon-source="static/images/icons/Bydleni-1.png"
>Bydlení 1</option>
<option
	data-image-source="static/images/icons/Bydleni-2-dark.png"
	data-image-icon-source="static/images/icons/Bydleni-2.png"
>Bydlení 2</option>
<option
	data-image-source="static/images/icons/Cestovni-ruch-1-dark.png"
	data-image-icon-source="static/images/icons/Cestovni-ruch-1.png"
>Cestovní ruch 1</option>
<option
	data-image-source="static/images/icons/Cestovni-ruch-2-dark.png"
	data-image-icon-source="static/images/icons/Cestovni-ruch-2.png"
>Cestovní ruch 2</option>
<option
	data-image-source="static/images/icons/Digitalizace-1-dark.png"
	data-image-icon-source="static/images/icons/Digitalizace-1.png"
>Digitalizace 1</option>
<option
	data-image-source="static/images/icons/Digitalizace-2-dark.png"
	data-image-icon-source="static/images/icons/Digitalizace-2.png"
>Digitalizace 2</option>
<option
	data-image-source="static/images/icons/Doprava-a-logistika-1-dark.png"
	data-image-icon-source="static/images/icons/Doprava-a-logistika-1.png"
>Doprava a logistika 1</option>
<option
	data-image-source="static/images/icons/Doprava-a-logistika-2-dark.png"
	data-image-icon-source="static/images/icons/Doprava-a-logistika-2.png"
>Doprava a logistika 2</option>
<option
	data-image-source="static/images/icons/Energetika-1-dark.png"
	data-image-icon-source="static/images/icons/Energetika-1.png"
>Energetika 1</option>
<option
	data-image-source="static/images/icons/Energetika-2-dark.png"
	data-image-icon-source="static/images/icons/Energetika-2.png"
>Energetika 2</option>
<option
	data-image-source="static/images/icons/Fajfka-dark.png"
	data-image-icon-source="static/images/icons/Fajfka.png"
>Fajfka</option>
<option
	data-image-source="static/images/icons/Finance-1-dark.png"
	data-image-icon-source="static/images/icons/Finance-1.png"
>Finance 1</option>
<option
	data-image-source="static/images/icons/Finance-2-dark.png"
	data-image-icon-source="static/images/icons/Finance-2.png"
>Finance 2</option>
<option
	data-image-source="static/images/icons/Jistoty-1-dark.png"
	data-image-icon-source="static/images/icons/Jistoty-1.png"
>Jistoty 1</option>
<option
	data-image-source="static/images/icons/Jistoty-2-dark.png"
	data-image-icon-source="static/images/icons/Jistoty-2.png"
>Jistoty 2</option>
<option
	data-image-source="static/images/icons/Kontrola-1-dark.png"
	data-image-icon-source="static/images/icons/Kontrola-1.png"
>Kontrola 1</option>
<option
	data-image-source="static/images/icons/Kontrola-2-dark.png"
	data-image-icon-source="static/images/icons/Kontrola-2.png"
>Kontrola 2</option>
<option
	data-image-source="static/images/icons/Kultura-1-dark.png"
	data-image-icon-source="static/images/icons/Kultura-1.png"
>Kultura 1</option>
<option
	data-image-source="static/images/icons/Kultura-2-dark.png"
	data-image-icon-source="static/images/icons/Kultura-2.png"
>Kultura 2</option>
<option
	data-image-source="static/images/icons/MHD-1-dark.png"
	data-image-icon-source="static/images/icons/MHD-1.png"
>MHD 1</option>
<option
	data-image-source="static/images/icons/MHD-2-dark.png"
	data-image-icon-source="static/images/icons/MHD-2.png"
>MHD 2</option>
<option
	data-image-source="static/images/icons/Mezinarodni-vztahy-1-dark.png"
	data-image-icon-source="static/images/icons/Mezinarodni-vztahy-1.png"
>Mezinarodní vztahy 1</option>
<option
	data-image-source="static/images/icons/Mezinarodni-vztahy-2-dark.png"
	data-image-icon-source="static/images/icons/Mezinarodni-vztahy-2.png"
>Mezinarodní vztahy 2</option>
<option
	data-image-source="static/images/icons/Moderni-mesto-1-dark.png"
	data-image-icon-source="static/images/icons/Moderni-mesto-1.png"
>Moderní město 1</option>
<option
	data-image-source="static/images/icons/Moderni-mesto-2-dark.png"
	data-image-icon-source="static/images/icons/Moderni-mesto-2.png"
>Moderní město 2</option>
<option
	data-image-source="static/images/icons/Modernizace-1-dark.png"
	data-image-icon-source="static/images/icons/Modernizace-1.png"
>Modernizace 1</option>
<option
	data-image-source="static/images/icons/Modernizace-2-dark.png"
	data-image-icon-source="static/images/icons/Modernizace-2.png"
>Modernizace 2</option>
<option
	data-image-source="static/images/icons/Newsletter-1-dark.png"
	data-image-icon-source="static/images/icons/Newsletter-1.png"
>Newsletter 1</option>
<option
	data-image-source="static/images/icons/Newsletter-2-dark.png"
	data-image-icon-source="static/images/icons/Newsletter-2.png"
>Newsletter 2</option>
<option
	data-image-source="static/images/icons/Obrana-1-dark.png"
	data-image-icon-source="static/images/icons/Obrana-1.png"
>Obrana 1</option>
<option
	data-image-source="static/images/icons/Obrana-2-dark.png"
	data-image-icon-source="static/images/icons/Obrana-2.png"
>Obrana 2</option>
<option
	data-image-source="static/images/icons/Odpadove-hospodarstvi-1-dark.png"
	data-image-icon-source="static/images/icons/Odpadove-hospodarstvi-1.png"
>Odpadové hospodářství 1</option>
<option
	data-image-source="static/images/icons/Odpadove-hospodarstvi-2-dark.png"
	data-image-icon-source="static/images/icons/Odpadove-hospodarstvi-2.png"
>Odpadové hospodářství 2</option>
<option
	data-image-source="static/images/icons/Otevrena-radnice-1-dark.png"
	data-image-icon-source="static/images/icons/Otevrena-radnice-1.png"
>Otevřená radnice 1</option>
<option
	data-image-source="static/images/icons/Otevrena-radnice-2-dark.png"
	data-image-icon-source="static/images/icons/Otevrena-radnice-2.png"
>Otevřená radnice 2</option>
<option
	data-image-source="static/images/icons/Pece-a-rodina-1-dark.png"
	data-image-icon-source="static/images/icons/Pece-a-rodina-1.png"
>Péče a rodina 1</option>
<option
	data-image-source="static/images/icons/Pece-a-rodina-2-dark.png"
	data-image-icon-source="static/images/icons/Pece-a-rodina-2.png"
>Péče a rodina 2</option>
<option
	data-image-source="static/images/icons/Prace-a-socialni-veci-1-dark.png"
	data-image-icon-source="static/images/icons/Prace-a-socialni-veci-1.png"
>Práce a sociální věci 1</option>
<option
	data-image-source="static/images/icons/Prace-a-socialni-veci-2-dark.png"
	data-image-icon-source="static/images/icons/Prace-a-socialni-veci-2.png"
>Práce a sociální věci 2</option>
<option
	data-image-source="static/images/icons/Program-1-dark.png"
	data-image-icon-source="static/images/icons/Program-1.png"
>Program 1</option>
<option
	data-image-source="static/images/icons/Program-2-dark.png"
	data-image-icon-source="static/images/icons/Program-2.png"
>Program 2</option>
<option
	data-image-source="static/images/icons/Prumysl-1-dark.png"
	data-image-icon-source="static/images/icons/Prumysl-1.png"
>Průmysl 1</option>
<option
	data-image-source="static/images/icons/Prumysl-2-dark.png"
	data-image-icon-source="static/images/icons/Prumysl-2.png"
>Průmysl 2</option>
<option
	data-image-source="static/images/icons/Sport-1-dark.png"
	data-image-icon-source="static/images/icons/Sport-1.png"
>Sport 1</option>
<option
	data-image-source="static/images/icons/Sport-2-dark.png"
	data-image-icon-source="static/images/icons/Sport-2.png"
>Sport 2</option>
<option
	data-image-source="static/images/icons/Spravedlnost-1-dark.png"
	data-image-icon-source="static/images/icons/Spravedlnost-1.png"
>Spravedlnost 1</option>
<option
	data-image-source="static/images/icons/Spravedlnost-2-dark.png"
	data-image-icon-source="static/images/icons/Spravedlnost-2.png"
>Spravedlnost 2</option>
<option
	data-image-source="static/images/icons/Srdce-1-dark.png"
	data-image-icon-source="static/images/icons/Srdce-1.png"
>Srdce 1</option>
<option
	data-image-source="static/images/icons/Srdce-2-dark.png"
	data-image-icon-source="static/images/icons/Srdce-2.png"
>Srdce 2</option>
<option
	data-image-source="static/images/icons/Svoboda-1-dark.png"
	data-image-icon-source="static/images/icons/Svoboda-1.png"
>Svoboda 1</option>
<option
	data-image-source="static/images/icons/Svoboda-2-dark.png"
	data-image-icon-source="static/images/icons/Svoboda-2.png"
>Svoboda 2</option>
<option
	data-image-source="static/images/icons/Transparentnost-1-dark.png"
	data-image-icon-source="static/images/icons/Transparentnost-1.png"
>Transparentnost 1</option>
<option
	data-image-source="static/images/icons/Transparentnost-2-dark.png"
	data-image-icon-source="static/images/icons/Transparentnost-2.png"
>Transparentnost 2</option>
<option
	data-image-source="static/images/icons/Uzemni-planovani-1-dark.png"
	data-image-icon-source="static/images/icons/Uzemni-planovani-1.png"
>Územní plánování 1</option>
<option
	data-image-source="static/images/icons/Uzemni-planovani-2-dark.png"
	data-image-icon-source="static/images/icons/Uzemni-planovani-2.png"
>Územní plánování 2</option>
<option
	data-image-source="static/images/icons/Vzdelavani-a-skolstvi-1-dark.png"
	data-image-icon-source="static/images/icons/Vzdelavani-a-skolstvi-1.png"
>Vzdělávání a školství 1</option>
<option
	data-image-source="static/images/icons/Vzdelavani-a-skolstvi-2-dark.png"
	data-image-icon-source="static/images/icons/Vzdelavani-a-skolstvi-2.png"
>Vzdělávání a školství 2</option>
<option
	data-image-source="static/images/icons/Worklife-balance-1-dark.png"
	data-image-icon-source="static/images/icons/Worklife-balance-1.png"
>Worklife balance 1</option>
<option
	data-image-source="static/images/icons/Worklife-balance-2-dark.png"
	data-image-icon-source="static/images/icons/Worklife-balance-2.png"
>Worklife balance 2</option>
<option
	data-image-source="static/images/icons/Zdravotnictvi-1-dark.png"
	data-image-icon-source="static/images/icons/Zdravotnictvi-1.png"
>Zdravotnictví 1</option>
<option
	data-image-source="static/images/icons/Zdravotnictvi-2-dark.png"
	data-image-icon-source="static/images/icons/Zdravotnictvi-2.png"
>Zdravotnictví 2</option>
<option
	data-image-source="static/images/icons/Zemedelstvi-1-dark.png"
	data-image-icon-source="static/images/icons/Zemedelstvi-1.png"
>Zemědělství 1</option>
<option
	data-image-source="static/images/icons/Zemedelstvi-2-dark.png"
	data-image-icon-source="static/images/icons/Zemedelstvi-2.png"
>Zemědělství 2</option>
<option
	data-image-source="static/images/icons/Zivotni-prostredi-1-dark.png"
	data-image-icon-source="static/images/icons/Zivotni-prostredi-1.png"
>Životní prostředí 1</option>
<option
	data-image-source="static/images/icons/Zivotni-prostredi-2-dark.png"
	data-image-icon-source="static/images/icons/Zivotni-prostredi-2.png"
>Životní prostředí 2</option>
`);

$("#pdf-a4").on(
	"click",
	async function(event) {
		await template.setResolution(Math.ceil(1122.52 * 3));

		const pdf = new jsPDF({
			orientation: "portrait",
			unit: "pt",
			format: "a4"
		});
		
		pdf.addImage(document.getElementById("graphicsCanvas"), "JPEG", 0, 0, 595.28, 841.89);
		
		pdf.save("VyhrajemA4.pdf");
		
		await template.setResolution(template.defaultResolution);
	}
);

$("#pdf-a3").on(
	"click",
	async function(event) {
		await template.setResolution(Math.ceil(1122.52 * 4));

		const pdf = new jsPDF({
			orientation: "portrait",
			unit: "pt",
			format: "a3"
		});
		
		pdf.addImage(document.getElementById("graphicsCanvas"), "JPEG", 0, 0, 841.89, 1190.55);
		
		pdf.save("VyhrajemA3.pdf");
		
		await template.setResolution(template.defaultResolution);
	}
);

$("#pdf-a2").on(
	"click",
	async function(event) {
		await template.setResolution(Math.ceil(1122.52 * 5));

		const pdf = new jsPDF({
			orientation: "portrait",
			unit: "pt",
			format: "a2"
		});
		
		pdf.addImage(document.getElementById("graphicsCanvas"), "JPEG", 0, 0, 1190.55, 1683.78);
		
		pdf.save("VyhrajemA2.pdf");
		
		await template.setResolution(template.defaultResolution);
	}
);

$("#pdf-a1").on(
	"click",
	async function(event) {
		await template.setResolution(Math.ceil(1122.52 * 6));

		const pdf = new jsPDF({
			orientation: "portrait",
			unit: "pt",
			format: "a1"
		});
		
		pdf.addImage(document.getElementById("graphicsCanvas"), "JPEG", 0, 0, 1683.78, 2383.94);
		
		pdf.save("VyhrajemA1.pdf");
		
		await template.setResolution(template.defaultResolution);
	}
);

$("#pdf-a0").on(
	"click",
	async function(event) {
		await template.setResolution(Math.ceil(1122.52 * 7));

		const pdf = new jsPDF({
			orientation: "portrait",
			unit: "pt",
			format: "a0"
		});
		
		pdf.addImage(document.getElementById("graphicsCanvas"), "JPEG", 0, 0, 2383.94, 3370.39);
		
		pdf.save("VyhrajemA0.pdf");
		
		await template.setResolution(template.defaultResolution);
	}
);

// Text

$("#primary-text").on(
	"input",
	async function(event) {
		await template.setPrimaryText(event.target.value);
	}
);

$("#secondary-text").on(
	"input",
	async function(event) {
		await template.setSecondaryText(event.target.value);
	}
);

$("#terciary-text").on(
	"input",
	async function(event) {
		await template.setTerciaryText(event.target.value);
	}
);

$("#url-text").on(
	"input",
	async function(event) {
		await template.setUrlText(event.target.value);
	}
);

$("#under-name-text").on(
	"input",
	async function(event) {
		await template.setUnderNameText(event.target.value);
	}
);

$("#secondary-under-name-text").on(
	"input",
	async function(event) {
		await template.setSecondaryUnderNameText(event.target.value);
	}
);

$("#date-text").on(
	"input",
	async function(event) {
		await template.setDateText(event.target.value);
	}
);

$("#time-text").on(
	"input",
	async function(event) {
		await template.setTimeText(event.target.value);
	}
);

$("#location-text").on(
	"input",
	async function(event) {
		await template.setLocationText(event.target.value);
	}
);

$("#name-text").on(
	"input",
	async function(event) {
		await template.setNameText(event.target.value);
	}
);

$("#secondary-name-text").on(
	"input",
	async function(event) {
		await template.setSecondaryNameText(event.target.value);
	}
);

// Location to top
$("#location-to-top").on(
	"change",
	async function(event) {
		await template.setLocationToTop(event.currentTarget.checked);
	}
);

// Show/hide requester text

$("#show-requester").on(
	"change",
	function(event) {
		if (event.currentTarget.checked) {
			$("#requester-text").css("display", "block");
			$("#requester-text").trigger("input");
		} else {
			$("#requester-text").css("display", "none");
			template.setRequesterText("");
		}
	}
);

$("#show-gradient").on(
	"change",
	async function(event) {
		await template.setGradient(event.currentTarget.checked);
	}
);

$("#requester-text").on(
	"input",
	async function(event) {
		await template.setRequesterText(event.target.value);
	}
);

$("#icon-text").on(
	"input",
	async function(event) {
		await template.setIconText(event.target.value);
	}
);

// QR codes
$("#qr-code-source").on(
	"input",
	function(event) {
		template.setQrCodeURI(event.target.value);
	}
);

// Image control

$("#resolution").on(
	"input",
	function(event) {
		let resolution = parseInt(event.target.value);
		
		if (Object.is(resolution, NaN) || resolution < 750) {
			resolution = 2000;
		}
		
		template.setResolution(resolution);
	}
);

// Images

$("#primary-image").on(
	"change",
	function(event) {
		if (event.target.files.length === 0) {
			return;
		}
		
		let uploadedImage = new Image();
		
		uploadedImage.onload = function() {
			if (this.width < 1080 || this.height < 1080) {
				alert("Prosím, nahraj obrázek s minimálním rozlišením 1080 pixelů na výšku i šířku.");
				$(event.target).val(null);
				
				return;
			}
			
			template.primaryImage = this;
			template.redrawCanvas();
		}
		
		uploadedImage.src = window.URL.createObjectURL(event.target.files[0]);
	}
);

$("#secondary-image").on(
	"change",
	function(event) {
		if (event.target.files.length === 0) {
			return;
		}
		
		let uploadedImage = new Image();
		
		uploadedImage.onload = function() {
			template.secondaryImage = this;
			template.redrawCanvas();
		}
		
		uploadedImage.src = window.URL.createObjectURL(event.target.files[0]);
	}
);

$("#logo-additional-image-reset").on(
	"click",
	function(event) {
		template.resetLogoImage();
		
		$("#logo-additional-image").val(null);
	}
);

$("#logo-additional-image").on(
	"change",
	function(event) {
		template.setLogoImageFromInput(event.target);
	}
);

$("#icon-image-input").on(
	"change",
	function(event) {
		template.setIconImageFromInput(event.target);
	}
);

$("#icon-image-reset").on(
	"click",
	function(event) {
		template.resetIconImage();
		
		$("#icon-image-input").val(null);
	}
);

$("#icon-image-selection").on(
	"select2:select",
	function(event) {
		let source = event.params.data.element.dataset.imageIconSource;
		
		if (source === undefined) {
			source = null;
		}
		
		template.setIconSource(source);
	}
);

// Point set
for (const num of Array(5).keys()) {
	$(`#5-point-set-text-${num + 1}`).on(
		"input",
		async function(event) {
			template.pointSet[num] = event.target.value;
			await template.redrawCanvas();
		}
	);
}

// Icon groups
for (let num of Array(4).keys()) {
	$(`#4-icon-group-image-${num + 1}-selection`).select2({
		templateSelection: formatSelect2ImageData,
		templateResult: formatSelect2ImageData,
		selectionCssClass: "select2-container-small-images",
		dropdownCssClass: "select2-container-small-images",
		width: "100%"
	});
	
	$(`#4-icon-group-image-${num + 1}-selection`).on(
		"select2:select",
		function(event) {
			let source = event.params.data.element.dataset.imageIconSource;
			
			if (source === undefined) {
				source = "";
			}
			
			template.iconSet[num].source = source;
			
			template.redrawCanvas();
		}
	);
	
	$(`#4-icon-group-image-${num + 1}-input`).on(
		"change",
		function(event) {
			if (event.target.files.length === 0) {
				template.iconSet[num].source = "";
				return;
			}
			
			const file = event.target.files[0];
			
			const reader = new FileReader();
			
			reader.onloadend = function() {
				template.iconSet[num].source = reader.result;
				template.redrawCanvas();
			};
			
			reader.readAsDataURL(file);
		}
	);
	
	$(`#4-icon-group-text-${num + 1}`).on(
		"input",
		function(event) {
			template.iconSet[num].text = event.target.value;
			
			template.redrawCanvas();
		}
	);
}

for (let num of Array(5).keys()) {
	$(`#5-icon-group-image-${num + 1}-selection`).select2({
		templateSelection: formatSelect2ImageData,
		templateResult: formatSelect2ImageData,
		selectionCssClass: "select2-container-small-images",
		dropdownCssClass: "select2-container-small-images",
		width: "100%"
	});
	
	$(`#5-icon-group-image-${num + 1}-selection`).on(
		"select2:select",
		function(event) {
			let source = event.params.data.element.dataset.imageIconSource;
			
			if (source === undefined) {
				source = "";
			}
			
			template.iconSet[num].source = source;
			
			template.redrawCanvas();
		}
	);
	
	$(`#5-icon-group-text-${num + 1}`).on(
		"input",
		function(event) {
			template.iconSet[num].text = event.target.value;
			
			template.redrawCanvas();
		}
	);
}

// Reaction groups
for (let num of Array(2).keys()) {
	$(`#2-reaction-set-image-${num + 1}-selection`).select2({
		templateSelection: formatSelect2ImageData,
		templateResult: formatSelect2ImageData,
		selectionCssClass: "select2-container-small-images",
		dropdownCssClass: "select2-container-small-images",
		width: "100%"
	});
	
	$(`#2-reaction-set-image-${num + 1}-selection`).on(
		"select2:select",
		function(event) {
			let source = event.params.data.element.dataset.imageReactionSource;
			
			if (source === undefined) {
				source = "";
			}
			
			template.reactions[num].source = source;
			
			template.redrawCanvas();
		}
	);
	
	$(`#2-reaction-set-text-${num + 1}`).on(
		"input",
		function(event) {
			template.reactions[num].text = event.target.value;
			
			template.redrawCanvas();
		}
	);
}

$("#location-image-selection").on(
	"select2:select",
	function(event) {
		let source = event.params.data.element.dataset.imageLocationSource;
		
		if (source === undefined) {
			source = null;
		}
		
		if (template.changeableAttributes.includes("locationImage")) {
			template.setLocationSource(source);
		}
	}
);

$("#location-image-selection").on(
	"change",
	function(event) {
		let source = $("#location-image-selection").select2("data")[0].element.dataset.imageLocationSource;
		
		if (source === undefined) {
			source = null;
		}
		
		if (template.changeableAttributes.includes("locationImage")) {
			template.setLocationSource(source);
		}
	}
);

$("#logo-is-center").on(
	"change",
	function(event) {
		template.setLogoIsCenter(event.target.checked);
	}
);

$("#background-has-pattern").on(
	"change",
	function(event) {
		template.setBackgroundHasPattern(event.target.checked);
	}
);

$("#location-image-selection").select2({
	templateSelection: formatSelect2ImageData,
	templateResult: formatSelect2ImageData,
	selectionCssClass: "select2-container-medium-images",
	dropdownCssClass: "select2-container-medium-images",
	width: "100%"
});

$("#icon-image-selection").select2({
	templateSelection: formatSelect2ImageData,
	templateResult: formatSelect2ImageData,
	selectionCssClass: "select2-container-small-images",
	dropdownCssClass: "select2-container-small-images",
	width: "100%"
});

// Images - positions & zoom

// Positions

$(".move-primary-image-left").on(
	"click",
	function(event) { template.movePrimaryImageLeft(); }
);

$(".move-primary-image-up").on(
	"click",
	function(event) { template.movePrimaryImageUp(); }
);

$(".move-primary-image-right").on(
	"click",
	function(event) { template.movePrimaryImageRight(); }
);

$(".move-primary-image-down").on(
	"click",
	function(event) { template.movePrimaryImageDown(); }
);

$(".move-primary-image-reset").on(
	"click",
	function(event) { template.resetPrimaryImagePosition(); }
);

// https://stackoverflow.com/a/15037102
// Thanks to markE!

let canvas = document.getElementById("graphicsCanvas");
let canvasOffset = $("#graphicsCanvas").offset();
let canvasBoundingClientRect = canvas.getBoundingClientRect();
let offsetX = canvasOffset.left;
let offsetY = canvasOffset.top;
let canvasWidth = canvas.width;
let canvasHeight = canvas.height;
let isDragging = false;

$(window).on("resize", function(){
	canvasOffset = $("#graphicsCanvas").offset();
	offsetX = canvasOffset.left;
	offsetY = canvasOffset.top;
	
	canvasBoundingClientRect = canvas.getBoundingClientRect();
});

$("#graphicsCanvas").on(
	"mousedown",
	function(event){
		canMouseX=parseInt(event.clientX-offsetX)
		canMouseY=parseInt(event.clientY-offsetY);
		// set the drag flag
		isDragging=true;
	}
);

$("#graphicsCanvas").on(
	"mousemove",
	function(event){
		canMouseX=parseInt(event.clientX-offsetX);
		canMouseY=parseInt(event.clientY-offsetY);
		// if the drag flag is set, clear the canvas and draw the image
		
		// Ignore movement when there is no image to move
		if (template === null || !template.changeableAttributes.includes("primaryImagePosition")) {
			return;
		}
		
		if (isDragging) {
			template.primaryImageX = (
				(
					canvas.width
					* (canMouseX / canvasBoundingClientRect.width)
				)
				- canvas.width / 2
			);
			template.primaryImageY = (
				(
					canvas.height
					* (canMouseY / canvasBoundingClientRect.height * template.aspectRatio)
				)
				- canvas.height / 2
			);
			
			template.redrawCanvas();
		}
	}
);

$("#graphicsCanvas").on(
	"mouseup",
	function(event){
		if (isDragging) {
			canMouseX=parseInt(event.clientX-offsetX);
			canMouseY=parseInt(event.clientY-offsetY);
			// clear the drag flag
			isDragging=false;
		}
	}
);

$("#graphicsCanvas").on(
	"mouseout",
	function(event){
		if (isDragging) {
			canMouseX=parseInt(event.clientX-offsetX);
			canMouseY=parseInt(event.clientY-offsetY);
			// clear the drag flag
			isDragging=false;
		}
	}
);

// Zoom

$(".zoom-primary-image-input").on(
	"input",
	function(event) { template.setPrimaryImageZoom(parseInt(event.target.value) * 0.01); }
);

$(".zoom-primary-image-reset").on(
	"click",
	function(event) {
		template.resetPrimaryImageZoom();
		$(".zoom-primary-image-input").val("100");
	}
);

$("#logo-image-zoom").on(
	"input",
	function(event) { template.setLogoImageZoom(parseInt(event.target.value) * 0.01); }
);

$("#logo-image-zoom-reset").on(
	"click",
	function(event) {
		template.resetLogoImageZoom();
		$("#logo-image-zoom").val("100");
	}
);

// Colors
$(".colorPickSelector").html(`<option
		value=\"#FFFFFF\"
	>Bílá</option>
	<option
		value=\"#000000\"
	>Černá</option>
	<option
		value=\"#CCCCCC\"
	>Šedá 1</option>
	<option
		value=\"#B3B3B3\"
	>Šedá 2</option>
	<option
		value=\"#868A89\"
	>Šedá 3</option>
	<option
		value=\"#4D4D4D\"
	>Šedá 4</option>
	<option
		value=\"#999999\"
	>Šedá 5</option>
	<option
		value=\"#838383\"
	>Šedá 6</option>
	<option
		value=\"#1D1D1B\"
	>Šedá 7</option>
	<option
		value=\"#962A51\"
	>Rudá 1</option>
	<option
		value=\"#712C45\"
	>Rudá 2</option>
	<option
		value=\"#4E1E2E\"
	>Rudá 3</option>
	<option
		value=\"#E63812\"
	>Rudá 4</option>
	<option
		value=\"#BE1E2D\"
	>Rudá 5</option>
	<option
		value=\"#DE2B58\"
	>Růžová 1</option>
	<option
		value=\"#B8284C\"
	>Růžová 2</option>
	<option
		value=\"#4E1E2F\"
	>Růžová 3</option>
	<option
		value=\"#DE8787\"
	>Růžová 4</option>
	<option
		value=\"#D35F5F\"
	>Růžová 5</option>
	<option
		value=\"#A02C2C\"
	>Růžová 6</option>
	<option
		value=\"#E17A85\"
	>Růžová 7</option>
	<option
		value=\"#AA5C65\"
	>Růžová 8</option>
	<option
		value=\"#864B52\"
	>Růžová 9</option>
	<option
		value=\"#FDA972\"
	>Pleťová 1</option>
	<option
		value=\"#ED8B4B\"
	>Pleťová 2</option>
	<option
		value=\"#B46C3D\"
	>Pleťová 3</option>
	<option
		value=\"#FFCC00\"
	>Žlutá 1</option>
	<option
		value=\"#D7B320\"
	>Žlutá 2</option>
	<option
		value=\"#A88D1C\"
	>Žlutá 3</option>
	<option
		value=\"#FFEDA5\"
	>Žlutá 4</option>
	<option
		value=\"#D8CC8E\"
	>Žlutá 5</option>
	<option
		value=\"#A8A573\"
	>Žlutá 6</option>
	<option
		value=\"#E2D7A9\"
	>Žlutá 7</option>
	<option
		value=\"#DAC465\"
	>Žlutá 8</option>
	<option
		value=\"#D7B31C\"
	>Žlutá 9</option>
	<option
		value=\"#FFD500\"
	>Žlutá 10</option>
	<option
		value=\"#FFDD55\"
	>Žlutá 11</option>
	<option
		value=\"#F6CB03\"
	>Žlutá 12</option>
	<option
		value=\"#CDDE87\"
	>Limetková 1</option>
	<option
		value=\"#ABC837\"
	>Limetková 2</option>
	<option
		value=\"#89A02C\"
	>Limetková 3</option>
	<option
		value=\"#D5FFD5\"
	>Limetková 4</option>
	<option
		value=\"#41DE87\"
	>Zelená 1</option>
	<option
		value=\"#209A37\"
	>Zelená 2</option>
	<option
		value=\"#206537\"
	>Zelená 3</option>
	<option
		value=\"#78BE43\"
	>Zelená 4</option>
	<option
		value=\"#AFE87E\"
	>Zelená 5</option>
	<option
		value=\"#50C450\"
	>Zelená 6</option>
	<option
		value=\"#8ED4A3\"
	>Zelená 7</option>
	<option
		value=\"#A9CE2D\"
	>Zelená 8</option>
	<option
		value=\"2DAD50\"
	>Zelená 9</option>
	<option
		value=\"#5ECCC9\"
	>Tyrkysová 1</option>
	<option
		value=\"#3DB6AC\"
	>Tyrkysová 2</option>
	<option
		value=\"#199887\"
	>Tyrkysová 3</option>
	<option
		value=\"#52D7CB\"
	>Tyrkysová 4</option>
	<option
		value=\"#5FBFB4\"
	>Tyrkysová 5</option>
	<option
		value=\"#32948B\"
	>Tyrkysová 6</option>
	<option
		value=\"#27756E\"
	>Tyrkysová 7</option>
	<option
		value=\"#3DB7AC\"
	>Tyrkysová 8</option>
	<option
		value=\"#26A4B7\"
	>Modrá 1</option>
	<option
		value=\"#0A4E5B\"
	>Modrá 2</option>
	<option
		value=\"#083F49\"
	>Modrá 3</option>
	<option
		value=\"#18C1DF\"
	>Modrá 4</option>
	<option
		value=\"#14A0B9\"
	>Modrá 5</option>
	<option
		value=\"#307684\"
	>Modrá 6</option>
	<option
		value=\"#123172\"
	>Modrá 7</option>
	<option
		value=\"#00173C\"
	>Modrá 8</option>
	<option
		value=\"#0038D1\"
	>Modrá 9</option>
	<option
		value=\"#AAEEFF\"
	>Světle modrá 1</option>
	<option
		value=\"#87CDDE\"
	>Světle modrá 2</option>
	<option
		value=\"#2C89A0\"
	>Světle modrá 3</option>
	<option
		value=\"#9796CA\"
	>Fialová 1</option>
	<option
		value=\"#585B88\"
	>Fialová 2</option>
	<option
		value=\"#21274E\"
	>Fialová 3</option>
	<option
		value=\"#DE87CD\"
	>Fialová 4</option>
	<option
		value=\"#D35FBC\"
	>Fialová 5</option>
	<option
		value=\"#782167\"
	>Fialová 6</option>
	<option
		value=\"#7B367C\"
	>Fialová 7</option>
	<option
		value=\"#5B305C\"
	>Fialová 8</option>
	<option
		value=\"#422A42\"
	>Fialová 9</option>
	<option
		value=\"#6E1646\"
	>Fialová 10</option>
	<option
		value=\"#3E2A5B\"
	>Fialová 11</option>
	<option
		value=\"#9796CA\"
	>Fialová 12</option>`);

$(".colorPickSelector").select2({
	templateSelection: formatSelect2ColorData,
	templateResult: formatSelect2ColorData,
	selectionCssClass: "select2-container-small-images",
	dropdownCssClass: "select2-container-small-images",
	width: "100%"
});

$(".colorPickSelector").on(
	"select2:select",
	function(event) {
		const attrName = snakeToCamel(event.target.id);
		const value = event.params.data.element.value;
		
		if (
			(attrName === "backgroundColor" && template.foregroundColor !== undefined && template.foregroundColor.toLowerCase() === value.toLowerCase()) ||
			(attrName === "foregroundColor" && template.backgroundColor !== undefined && template.backgroundColor.toLowerCase() === value.toLowerCase())
		) {
			alert("Silně nedoporučujeme používat stejnou barvu pro pozadí i popředí, šablony na to nejsou dělané. Nezastavíme tě, ale zkus si to prosím ještě promyslet.");
		}
		
		template[attrName] = value;
		template.redrawCanvas();
	}
);

$("#coalition-selection").select2({
	templateSelection: formatSelect2ImageData,
	templateResult: formatSelect2ImageData,
	selectionCssClass: "select2-container-medium-images",
	dropdownCssClass: "select2-container-medium-images",
	width: "100%"
});

let coalitionName = null;


async function setCoalition(coalition, template) {
	if (coalition === null || coalition === undefined || coalition === "no-coalition") {
		coalitionName = null;
		
		// reset
		template.setLightLogoSource(template.lightDefaultLogoSource, true);
		template.setDarkLogoSource(template.darkDefaultLogoSource, true);
		
		$("#location-image-selection").val("Žádná lokalizace");
		$("#location-image-selection").trigger("change");
		
		initTemplate($("#template-selection").select2("data")[0].element.dataset.templateType);
		
		return;
	}
	
	coalitionName = coalition;
	
	function setGlobalSpecifics(coalition) {
		switch (coalition) {
			case "louny-spolecne":
				if (template.changeableAttributes.includes("logoImage")) {
					template.setDarkLogoSource("static/images/logos/louny_spolecne-dark.png", true);
					template.setLightLogoSource("static/images/logos/louny_spolecne.png", true);
				} else if (template.changeableAttributes.includes("verticalLogoImage")) {
					template.setDarkLogoSource("static/images/vertical-logos/Louny-dark.png", true);
					template.setLightLogoSource("static/images/vertical-logos/Louny-light.png", true);
				}
				
				if (template.changeableAttributes.includes("locationImage")) {
					$("#location-image-selection").val("Louny");
					$("#location-image-selection").trigger("change");
				}
				
				break;
		}
	}
	
	template.primaryFont = "'Bebas Neue'";
	template.primaryFontStyle = "";
	template.primaryTextTransformationHook = function(text) { return text; }

	switch(coalition) {
		case "louny-spolecne-purple-dark":
			await template.setPrimaryColorScheme("louny-spolecne-white-on-purple", true);
			
			setGlobalSpecifics("louny-spolecne");
			
			reloadColorPalette();
			
			break;
		case "louny-spolecne-purple-light":
			await template.setPrimaryColorScheme("louny-spolecne-black-on-white", true);
			
			setGlobalSpecifics("louny-spolecne");
			
			reloadColorPalette();
			
			break;
		case "louny-spolecne-dark":
			await template.setPrimaryColorScheme("black-on-white", true);
			
			setGlobalSpecifics("louny-spolecne");
			
			reloadColorPalette();
			
			break;
	}
	
	await template.redrawCanvas();
}

$("#coalition-selection").on(
	"select2:select",
	function(event) {
		setCoalition(event.params.data.id, template);
	}
);

$("#coalition-selection").on(
	"change",
	function(event) {
		setCoalition(event.currentTarget.value, template);
	}
);

// Templates

$("#template-selection").select2({
	templateSelection: formatSelect2ImageData,
	templateResult: formatSelect2ImageData,
	selectionCssClass: "select2-container-medium-images",
	dropdownCssClass: "select2-container-medium-images",
	width: "100%"
});

$("#template-selection").on(
	"select2:select",
	function(event) {
		templateName = event.params.data.element.dataset.templateType;
		initTemplate(event.params.data.element.dataset.templateType);
	}
);

// https://stackoverflow.com/a/50300880
// Thanks to Ulf Aslak!
$("#download").on(
	"click",
	function(){
		checkMissingImage(template);
		
		let link = document.createElement('a');

		link.download = "Vyhrajem.png";
		link.href = document.getElementById("graphicsCanvas").toDataURL()

		link.click();
	}
);

var templateName = null;

// Run
$(window).ready(
	async function () {
		// Make absolutely sure Bebas Neue is loaded.
		// Quick fix
		await (
			fetch("static/fonts/bebas-neue/BebasNeue.woff")
			.then(resp => resp.arrayBuffer())
			.then(font => {
				const fontFace = new FontFace("Bebas Neue", font);
				document.fonts.add(fontFace);
			})
		);
		
		templateName = $("#template-selection").select2("data")[0].element.dataset.templateType;
		
		if ($("#show-requester").is(":checked")) {
			$("#requester-text").css("display", "block");
		}

		initTemplate(templateName);
	}
);
